@@ -2,8 +2,7 @@ test_that("gaze heatmap function works correctly", {
   skip_if_not_installed("MASS")
   skip_if_not_installed("viridis")
 
-  sample_data <- eyelink_asc_demo_dataset() |>
-    glassbox()
+  sample_data <- eyelink_asc_demo_dataset() |> glassbox()
 
   # test plot_gaze_heatmap function on monocular data
   expect_silent({
@@ -26,8 +25,7 @@ test_that("gaze heatmap function works correctly", {
     )
   })
 
-  sample_data <- eyelink_asc_binocular_demo_dataset() |>
-    glassbox()
+  sample_data <- eyelink_asc_binocular_demo_dataset() |> glassbox()
 
   # test plot_gaze_heatmap function on binocular data
   expect_silent({
